#!/bin/bash

# SF Mirror
if [[ ${SF_MIRROR} == "" ]]; then
    SF_MIRROR="easynews."
fi

# MinGW32 Compiler
MINGW_GCC_VERSION=3.4.2-20040916-1
MINGW_W32API_VERSION=3.7
MINGW_RUNTIME_VERSION=3.9
MINGW_BINUTILS_VERSION=2.16.91-20060119-1
MINGW_GCC=http://${SF_MIRROR}dl.sourceforge.net/sourceforge/mingw/gcc-core-${MINGW_GCC_VERSION}.tar.gz
MINGW_GCC_DIR=libexec/gcc
MINGW_W32API=http://${SF_MIRROR}dl.sourceforge.net/sourceforge/mingw/w32api-${MINGW_W32API_VERSION}.tar.gz
MINGW_W32API_DIR=include
MINGW_RUNTIME=http://${SF_MIRROR}dl.sourceforge.net/sourceforge/mingw/mingw-runtime-${MINGW_RUNTIME_VERSION}.tar.gz
MINGW_RUNTIME_DIR=doc/mingw-runtime
MINGW_BINUTILS=http://${SF_MIRROR}dl.sourceforge.net/sourceforge/mingw/binutils-${MINGW_BINUTILS_VERSION}.tar.gz
MINGW_BINUTILS_DIR=mingw32

# GTK+ Development Package
GTK_VERSION=2.10.6
GTK_REVISION=1
GTK_DEV=http://${SF_MIRROR}dl.sourceforge.net/sourceforge/gladewin32/gtk-dev-${GTK_VERSION}-win32-${GTK_REVISION}.exe
GTK_DIR=GTK

# WinPcap
PCAP_VERSION=3_1
PCAP_DEV=http://www.winpcap.org/install/bin/WpdPack_${PCAP_VERSION}.zip
PCAP_INSTALL=http://www.winpcap.org/install/bin/WinPcap_${PCAP_VERSION}.exe
PCAP_DIR=wpdpack

# libdnet
DNET_VERSION=1.11
DNET_DEV=http://${SF_MIRROR}dl.sourceforge.net/sourceforge/libdnet/libdnet-${DNET_VERSION}-win32.zip
DNET_DIR=libdnet

# Dependencies Necessary to Build the Aimject Installer
NSIS_VERSION=2.21
NSIS=http://${SF_MIRROR}dl.sourceforge.net/sourceforge/nsis/nsis-${NSIS_VERSION}-setup.exe
NSIS_REG_KEY="/proc/registry/HKEY_LOCAL_MACHINE/SOFTWARE/Microsoft/Windows/CurrentVersion/Uninstall/NSIS/InstallLocation"

# Aimject Source Code Package
export AIMJECT=http://jon.oberheide.org/projects/aimject/downloads/aimject-VERSION.tar.gz;

echo "Aimject Win32 Builder"

export OS=`uname -o`;
if [[ ${OS} != "Cygwin" ]] ; then
    echo;
    echo "ERROR: This script is meant to be run on the Cygwin platform.";
    echo;
fi

export WGET=`which wget`;
export TAR=`which tar`;
export GUNZIP=`which gunzip`;
export UNZIP5=`which unzip`;
export SED=`which sed`;

if [[ ${WGET} == "" ]] || [[ ${TAR} == "" ]] || [[ ${GUNZIP} == "" ]] ||
   [[ ${UNZIP5} == "" ]] || [[ ${SED} == "" ]]; then
    echo
    echo "ERROR: One or more required utilities were not found.";
    echo "Use Cygwin\'s setup.exe to install all packages listed above."
    exit;
fi

function fetch () {
    local PKG_URL=$1;
    local PKG_FN=`echo $1 | ${SED} s#.*/##`;
    local NO_DIE=$2;
    echo "  Fetching ${PKG_FN}...";
    if [ ! -e ${PKG_FN} ] ; then
        ${WGET} -nv -t4 ${PKG_URL} 2>&1 | ${SED} s/.*\\\(-\>\\\)/\\1/;
    fi
    if [[ ${NO_DIE} != TRUE ]]; then
        if [ ! -r ${PKG_FN} ]; then
            echo "  * ERROR: Failed fetching ${PKG_FN}";
            exit;
        fi
    fi
}

function install () {
    local PKG_NAME=$1;
    local PKG_FN=`echo $2 | ${SED} s#.*/##`;
    local PKG_ARGS=$3;

    echo "  Installing ${PKG_NAME} (${PKG_FN})...";

    # Make the installer executable
    chmod a+x ${AIMJECT_TEMP_DIR}/${PKG_FN};
    ${AIMJECT_TEMP_DIR}/${PKG_FN} ${PKG_ARGS};
}

function pathify () {
    PATHIFIED_PATH=`echo "$1" | ${SED} "s%\(.\):%/cygdrive/\1%; s#\\\\\#/#g; s#Program Files#Progra~1#"`;
}

function extract () {
    local PKG_NAME=$1;
    local PKG_FN=`echo $2 | ${SED} s#.*/##`;
    local PKG_TYPE=`echo ${PKG_FN} | ${SED} "s#.*\\\(\\\.tar\\\.gz\|\\\.tar\\\.bz2\|\\\.zip\\\)#\\1#"`;

    # Figure out which utility to use to extract this package.
    if [[ ${PKG_TYPE} == ".tar.gz" ]] ; then
        local PKG_CMD="${TAR} -xzf";
    elif [[ ${PKG_TYPE} == ".tar.bz2" ]] ; then
        local PKG_CMD="${TAR} -xjf";
    else
        local PKG_CMD="${UNZIP5} -q";
    fi

    echo "  ${PKG_NAME} package (${PKG_FN})...";
    echo "  * Unpacking ${PKG_NAME} package...";
    ${PKG_CMD} "${AIMJECT_TEMP_DIR}/${PKG_FN}";
}

echo;
echo "Preparing a Aimject build environment..."

rm -rf temp;
mkdir -p temp;
cd temp;

echo "Downloading package files to temp directory..."

fetch ${MINGW_GCC};
fetch ${MINGW_W32API};
fetch ${MINGW_RUNTIME};
fetch ${MINGW_BINUTILS};
fetch ${NSIS};
fetch ${PCAP_DEV};
fetch ${PCAP_INSTALL};
fetch ${DNET_DEV};
fetch ${GTK_DEV};

echo;
cd ..;
AIMJECT_TEMP_DIR=temp;

echo "Beginning required installations...";

install "NullSoft Install System" ${NSIS} "/S";
install "GTK+ Devel Package" ${GTK_DEV} "/S";

echo;
echo "Extracting development packages...";

rm -rf build;
mkdir -p build;
cd build;
AIMJECT_TEMP_DIR=../temp;

pathify "c:\GTK";
GTK_FILE=`echo ${GTK_DEV} | ${SED} s#.*/##`;
cp -rf ${PATHIFIED_PATH} ./

extract "libdnet" "${DNET_DEV}";
mv libdnet-${DNET_VERSION}-win32/ ${DNET_DIR}/

extract "WinPcap" "${PCAP_DEV}";

PCAP_FILE=`echo ${PCAP_INSTALL} | ${SED} s#.*/##`;
cp ${AIMJECT_TEMP_DIR}/${PCAP_FILE} ./

mkdir -p mingw;
cd mingw;
AIMJECT_TEMP_DIR=../../temp;

extract "MinGW GCC" "${MINGW_GCC}";
extract "MinGW W32API" "${MINGW_W32API}";
extract "MinGW Runtime" "${MINGW_RUNTIME}";
extract "MinGW BinUtils" "${MINGW_BINUTILS}";

cd ..;
cd ..;
AIMJECT_TEMP_DIR=temp;

echo;
echo "Running ./configure...";
PKG_CONFIG_PATH="./build/${GTK_DIR}/lib/pkgconfig" PATH="./build/${GTK_DIR}/bin:./build/mingw/bin:${PATH}" ./configure

echo;
echo "Running make...";
PATH="./build/mingw/bin:${PATH}" make

echo;
echo "Creating installer...";
NSIS_PATH=`cat ${NSIS_REG_KEY}`;
pathify "${NSIS_PATH}";
PATH="${PATHIFIED_PATH}:${PATH}" makensis data/aimject.nsi

rm -rf temp;
rm -rf build;

echo;
echo "DONE!";
echo;
