/*
 * Aimject
 *
 * Copyright (C) 2006 Jon Oberheide <jon@oberheide.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <config.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <glib.h>
#include <dnet.h>
#include <pcap.h>
#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <glade/glade.h>

#include "os.h"
#include "arpspoof.h"
#include "dnsspoof.h"
#include "interface.h"
#include "conn.h"
#include "aim.h"
#include "util.h"
#include "main.h"

#define GLADE G_DIR_SEPARATOR_S "aimject.glade"

extern aimject_t *aj;

client_t *
get_selected_client()
{
	client_t *client;
	GtkTreeIter iter;
	GtkTreeModel *model;
	GtkTreeSelection *select;

	select = gtk_tree_view_get_selection(GTK_TREE_VIEW(aj->client_list));

	g_return_val_if_fail(gtk_tree_selection_get_selected(select, &model, &iter), NULL);

	gtk_tree_model_get(model, &iter, 1, &client, -1);

	g_assert(client != NULL);

	return client;
}

conv_t *
get_selected_conv(client_t *client)
{
	conv_t *conv;
	GtkTreeIter iter;
	GtkTreeModel *model;
	GtkTreeSelection *select;

	g_assert(client != NULL);

	select = gtk_tree_view_get_selection(GTK_TREE_VIEW(aj->conv_list));

	g_return_val_if_fail(gtk_tree_selection_get_selected(select, &model, &iter), NULL);

	gtk_tree_model_get(model, &iter, 1, &conv, -1);

	g_assert(conv != NULL);

	return conv;
}

void
interface_set_conv(client_t *client, conv_t *conv)
{
	gchar *out = g_strdup_printf("%s -> %s", client->sn_norm, conv->sn_norm);
	gchar *in = g_strdup_printf("%s -> %s", conv->sn_norm, client->sn_norm);

	/* enable inject buttons */
	gtk_widget_set_sensitive(aj->inject_button, TRUE);
	gtk_widget_set_sensitive(aj->inject_radio1, TRUE);
	gtk_widget_set_sensitive(aj->inject_radio2, TRUE);
	gtk_button_set_label(GTK_BUTTON(aj->inject_radio1), out);
	gtk_button_set_label(GTK_BUTTON(aj->inject_radio2), in);

	/* enable mute buttons */
	gtk_widget_set_sensitive(aj->mute_to_conv_button, TRUE);
	gtk_widget_set_sensitive(aj->mute_from_conv_button, TRUE);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(aj->mute_to_conv_button), conv->mute_to);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(aj->mute_from_conv_button), conv->mute_from);

	g_free(out);
	g_free(in);
}

void
interface_reset_conv()
{
	GtkTextIter start, end;

	/* clear out message box */
	gtk_text_buffer_get_bounds(aj->msg_buffer, &start, &end);
	gtk_text_buffer_delete(aj->msg_buffer, &start, &end);

	/* disable inject buttons */
	gtk_widget_set_sensitive(aj->inject_button, FALSE);
	gtk_widget_set_sensitive(aj->inject_radio1, FALSE);
	gtk_widget_set_sensitive(aj->inject_radio2, FALSE);
	gtk_button_set_label(GTK_BUTTON(aj->inject_radio1), "");
	gtk_button_set_label(GTK_BUTTON(aj->inject_radio2), "");

	/* disable mute buttons */
	gtk_widget_set_sensitive(aj->mute_to_conv_button, FALSE);
	gtk_widget_set_sensitive(aj->mute_from_conv_button, FALSE);
}

void
interface_set_client(client_t *client)
{
	guint i;
	conv_t *conv;
	GtkTreeIter iter;

	g_assert(client != NULL);

	/* enable conv list buttons */
	gtk_widget_set_sensitive(aj->add_conv_button, TRUE);
	gtk_widget_set_sensitive(aj->delete_conv_button, TRUE);

	/* clear out conv list store */
	gtk_list_store_clear(aj->conv_store);

	/* insert each conversation into the list */
	for (i = 0; i < client->convs->len; ++i) {
		conv = g_ptr_array_index(client->convs, i);
		g_assert(conv != NULL);

		gtk_list_store_append(aj->conv_store, &iter);
		gtk_list_store_set(aj->conv_store, &iter, 0, conv->sn_norm, 1, conv, -1);
	}
}

void
interface_reset_client()
{
	GtkTreeIter iter;

	/* disable conv list buttons */
	gtk_widget_set_sensitive(aj->add_conv_button, FALSE);
	gtk_widget_set_sensitive(aj->delete_conv_button, FALSE);

	/* clear out conv list store */
	gtk_list_store_clear(aj->conv_store);

	/* clear stats */
	gtk_tree_model_get_iter_first(GTK_TREE_MODEL(aj->stats_store), &iter);
	gtk_list_store_set(GTK_LIST_STORE(aj->stats_store), &iter, 1, "", -1);
	gtk_tree_model_iter_next(GTK_TREE_MODEL(aj->stats_store), &iter);
	gtk_list_store_set(GTK_LIST_STORE(aj->stats_store), &iter, 1, "", -1);
	gtk_tree_model_iter_next(GTK_TREE_MODEL(aj->stats_store), &iter);
	gtk_list_store_set(GTK_LIST_STORE(aj->stats_store), &iter, 1, "", -1);
	gtk_tree_model_iter_next(GTK_TREE_MODEL(aj->stats_store), &iter);
	gtk_list_store_set(GTK_LIST_STORE(aj->stats_store), &iter, 1, "", -1);
	gtk_tree_model_iter_next(GTK_TREE_MODEL(aj->stats_store), &iter);
	gtk_list_store_set(GTK_LIST_STORE(aj->stats_store), &iter, 1, "", -1);
	gtk_tree_model_iter_next(GTK_TREE_MODEL(aj->stats_store), &iter);
	gtk_list_store_set(GTK_LIST_STORE(aj->stats_store), &iter, 1, "", -1);
	gtk_tree_model_iter_next(GTK_TREE_MODEL(aj->stats_store), &iter);
	gtk_list_store_set(GTK_LIST_STORE(aj->stats_store), &iter, 1, "", -1);
	gtk_tree_model_iter_next(GTK_TREE_MODEL(aj->stats_store), &iter);
	gtk_list_store_set(GTK_LIST_STORE(aj->stats_store), &iter, 1, "", -1);
	gtk_tree_model_iter_next(GTK_TREE_MODEL(aj->stats_store), &iter);
	gtk_list_store_set(GTK_LIST_STORE(aj->stats_store), &iter, 1, "", -1);
	gtk_tree_model_iter_next(GTK_TREE_MODEL(aj->stats_store), &iter);
	gtk_list_store_set(GTK_LIST_STORE(aj->stats_store), &iter, 1, "", -1);
	gtk_tree_model_iter_next(GTK_TREE_MODEL(aj->stats_store), &iter);
	gtk_list_store_set(GTK_LIST_STORE(aj->stats_store), &iter, 1, "", -1);
}

void
interface_reset_all()
{
	interface_reset_client();
	interface_reset_conv();
}

void
interface_insert_buffer(client_t *client, conv_t *conv, msg_t *msg, GtkTextIter *iter)
{
	gchar *tmp, *timestamp, *sn, *type;

	g_return_if_fail(msg != NULL);

	/* timestamp formatting */
	tmp = ctime(&msg->timestamp);
	tmp += 4;
	tmp[strlen(tmp)-6] = '\0';
	timestamp = g_strdup_printf("(%s) ", tmp);

	/* special message type tags */
	switch (msg->type) {
		case MSG_AUTO_RESP:
			type = g_strdup("AUTO-RESP ");
		break;

		case MSG_INJECT:
			type = g_strdup("INJECT ");
		break;

		case MSG_MUTED:
			type = g_strdup("MUTED ");
		break;

		default:
			type = g_strdup("");
		break;
	}

	if (msg->dir == DIR_OUT) {
		sn = g_strdup_printf("%s: ", client->sn_norm);

		gtk_text_buffer_insert_with_tags_by_name(aj->msg_buffer, iter, timestamp, strlen(timestamp), "blue_foreground", NULL);
		gtk_text_buffer_insert_with_tags_by_name(aj->msg_buffer, iter, type, strlen(type), "blue_foreground", "bold", NULL);
		gtk_text_buffer_insert_with_tags_by_name(aj->msg_buffer, iter, sn, strlen(sn), "blue_foreground", "bold", NULL);
	} else {
		sn = g_strdup_printf("%s: ", conv->sn_norm);

		gtk_text_buffer_insert_with_tags_by_name(aj->msg_buffer, iter, timestamp, strlen(timestamp), "red_foreground", NULL);
		gtk_text_buffer_insert_with_tags_by_name(aj->msg_buffer, iter, type, strlen(type), "red_foreground", "bold", NULL);
		gtk_text_buffer_insert_with_tags_by_name(aj->msg_buffer, iter, sn, strlen(sn), "red_foreground", "bold", NULL);
	}

	gtk_text_buffer_insert(aj->msg_buffer, iter, msg->data, strlen(msg->data));
	gtk_text_buffer_insert(aj->msg_buffer, iter, "\n", strlen("\n"));

	g_free(sn);
	g_free(type);
	g_free(timestamp);
}

void
interface_refresh_stats(client_t *client)
{
	GtkTreeIter iter;

	gchar *new_ip = g_strdup_printf("%s:%d", inet_ntoa(client->new_conn->addr.sin_addr), client->new_conn->addr.sin_port);
	gchar *bos_ip = g_strdup_printf("%s:%d", inet_ntoa(client->bos_conn->addr.sin_addr), client->bos_conn->addr.sin_port);
	gchar *conv_count = g_strdup_printf("%d", client->conv_count);
	gchar *msg_recv_count = g_strdup_printf("%d", client->msg_recv_count);
	gchar *msg_sent_count = g_strdup_printf("%d", client->msg_sent_count);
	gchar *last_recv_msg = g_strdup((client->last_recv_msg == 0 ? "N/A" : ctime(&client->last_recv_msg)));
	gchar *last_sent_msg = g_strdup((client->last_sent_msg == 0 ? "N/A" : ctime(&client->last_sent_msg)));
	gchar *seq_in = g_strdup_printf("%d", client->last_seq_in);
	gchar *seq_out = g_strdup_printf("%d", client->last_seq_out);

	/* per-conversation statistics */
	gchar *sn_fmt, *font;
	conv_t *conv = get_selected_conv(client);
	if (conv) {
		sn_fmt = g_strdup((conv->sn_fmt ? conv->sn_fmt : "N/A"));
		font = g_strdup((conv->font ? conv->font : "N/A"));
	} else {
		sn_fmt = g_strdup("N/A");
		font = g_strdup("N/A");
	}

	/* get first iter */
	gtk_tree_model_get_iter_first(GTK_TREE_MODEL(aj->stats_store), &iter);

	/* refresh each statistic */
	gtk_list_store_set(GTK_LIST_STORE(aj->stats_store), &iter, 1, new_ip, -1);
	gtk_tree_model_iter_next(GTK_TREE_MODEL(aj->stats_store), &iter);
	gtk_list_store_set(GTK_LIST_STORE(aj->stats_store), &iter, 1, bos_ip, -1);
	gtk_tree_model_iter_next(GTK_TREE_MODEL(aj->stats_store), &iter);
	gtk_list_store_set(GTK_LIST_STORE(aj->stats_store), &iter, 1, conv_count, -1);
	gtk_tree_model_iter_next(GTK_TREE_MODEL(aj->stats_store), &iter);
	gtk_list_store_set(GTK_LIST_STORE(aj->stats_store), &iter, 1, msg_recv_count, -1);
	gtk_tree_model_iter_next(GTK_TREE_MODEL(aj->stats_store), &iter);
	gtk_list_store_set(GTK_LIST_STORE(aj->stats_store), &iter, 1, msg_sent_count, -1);
	gtk_tree_model_iter_next(GTK_TREE_MODEL(aj->stats_store), &iter);
	gtk_list_store_set(GTK_LIST_STORE(aj->stats_store), &iter, 1, last_recv_msg, -1);
	gtk_tree_model_iter_next(GTK_TREE_MODEL(aj->stats_store), &iter);
	gtk_list_store_set(GTK_LIST_STORE(aj->stats_store), &iter, 1, last_sent_msg, -1);
	gtk_tree_model_iter_next(GTK_TREE_MODEL(aj->stats_store), &iter);
	gtk_list_store_set(GTK_LIST_STORE(aj->stats_store), &iter, 1, seq_in, -1);
	gtk_tree_model_iter_next(GTK_TREE_MODEL(aj->stats_store), &iter);
	gtk_list_store_set(GTK_LIST_STORE(aj->stats_store), &iter, 1, seq_out, -1);
	gtk_tree_model_iter_next(GTK_TREE_MODEL(aj->stats_store), &iter);
	gtk_list_store_set(GTK_LIST_STORE(aj->stats_store), &iter, 1, sn_fmt, -1);
	gtk_tree_model_iter_next(GTK_TREE_MODEL(aj->stats_store), &iter);
	gtk_list_store_set(GTK_LIST_STORE(aj->stats_store), &iter, 1, font, -1);

	g_free(font);
	g_free(sn_fmt);
	g_free(seq_out);
	g_free(seq_in);
	g_free(last_recv_msg);
	g_free(last_sent_msg);
	g_free(msg_recv_count);
	g_free(msg_sent_count);
	g_free(conv_count);
	g_free(bos_ip);
	g_free(new_ip);
}

void
interface_msg_refresh(client_t *client, conv_t *conv)
{
	guint i;
	msg_t *msg;
	GtkTextMark *mark;
	GtkTextIter iter, start_iter, end_iter;

	g_assert(client != NULL);
	g_assert(conv != NULL);

	gtk_text_buffer_get_bounds(aj->msg_buffer, &start_iter, &end_iter);
	gtk_text_buffer_delete(aj->msg_buffer, &start_iter, &end_iter);
	gtk_text_buffer_get_start_iter(aj->msg_buffer, &iter);

	for (i = 0; i < conv->msgs->len; ++i) {
		msg = g_ptr_array_index(conv->msgs, i);

		/* insert message into the buffer */
		interface_insert_buffer(client, conv, msg, &iter);
	}

	/* scroll to bottom of text view */
	gtk_text_buffer_get_bounds(aj->msg_buffer, &start_iter, &end_iter);
	mark = gtk_text_buffer_create_mark(aj->msg_buffer, NULL, &end_iter, FALSE);
	gtk_text_view_scroll_mark_onscreen(GTK_TEXT_VIEW(aj->msg_data), mark);
	gtk_text_buffer_delete_mark(aj->msg_buffer, mark);
}

void
interface_process_inject()
{
	gint direction;
	gchar *text;
	gboolean ret;
	GtkTextIter start, end;

	msg_t *msg;
	conv_t *conv;
	client_t *client;

	g_debug("process inject triggered!");

	/* verify we have a selected user and conv */
	client = get_selected_client();
	g_return_if_fail(client != NULL);
	conv = get_selected_conv(client);
	g_return_if_fail(conv != NULL);

	/* retrieve entered text */
	gtk_text_buffer_get_bounds(aj->inject_buffer, &start, &end);
	text = gtk_text_buffer_get_text(aj->inject_buffer, &start, &end, FALSE);
	g_return_if_fail(text != NULL);
	g_return_if_fail(strlen(text) > 0);

	/* determine which direction to inject */
	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(aj->inject_radio1))) {
		direction = DIR_OUT;
	} else {
		direction = DIR_IN;
	}

	g_debug("inject in direction: %d", direction);
	ret = inject_aim_msg(client, conv, text, direction);
	if (ret) {
		g_debug("injection success");
		msg = g_new0(msg_t, 1);
		msg->dir = direction;
		msg->type = MSG_INJECT;
		msg->timestamp = time(NULL);
		msg->data = g_strdup(text);

		main_add_msg(client, conv, msg);
		interface_add_msg(client, conv, msg);
	} else {
		g_debug("error during injection");
	}

	/* clear out text box */
	gtk_text_buffer_delete(aj->inject_buffer, &start, &end);

	g_free(text);
}

EXPORT gboolean
cb_inject_data_key_press_event(GtkWidget *widget, GdkEventKey *event, gpointer data)
{
	if ((event->keyval == GDK_Return || event->keyval == GDK_KP_Enter) &&
	    (event->state & gtk_accelerator_get_default_mod_mask()) == 0) {
		interface_process_inject();
		return TRUE;
	}
	return FALSE;
}

EXPORT void
cb_delete_conv_button_clicked(GtkButton *button, gpointer data)
{
	conv_t *conv;
	client_t *client;
	GtkTreeIter iter;
	GtkTreeModel *model;
	GtkTreeSelection *select;

	/* verify we have a selected user and conv */
	client = get_selected_client();
	g_return_if_fail(client != NULL);
	conv = get_selected_conv(client);
	g_return_if_fail(conv != NULL);

	/* remove conv from interface */
	select = gtk_tree_view_get_selection(GTK_TREE_VIEW(aj->conv_list));
	g_return_if_fail(gtk_tree_selection_get_selected(select, &model, &iter));
	gtk_list_store_remove(aj->conv_store, &iter);

	/* reset conv interface */
	interface_reset_conv();

	/* remove conv from data store */
	main_destroy_conv(client, conv);

	/* refresh the statistics window */
	interface_refresh_stats(client);
}

EXPORT void
cb_add_conv_button_clicked(GtkButton *button, gpointer data)
{
	gtk_entry_set_text(GTK_ENTRY(aj->input_screenname_entry), "");
	gtk_widget_show(aj->input_window);
	gtk_widget_grab_focus(aj->input_screenname_entry);
}

EXPORT void
cb_input_cancel_clicked(GtkButton *button, gpointer data)
{
	gtk_widget_hide(aj->input_window);
}

EXPORT void
cb_input_ok_clicked(GtkButton *button, gpointer data)
{
	conv_t *conv;
	client_t *client;
	gchar *sn_fmt, *sn_norm;

	gtk_widget_hide(aj->input_window);

	/* verify we have a selected user */
	client = get_selected_client();
	g_return_if_fail(client != NULL);

	sn_fmt = (gchar *) gtk_entry_get_text(GTK_ENTRY(aj->input_screenname_entry));
	sn_norm = str_normalize(sn_fmt);

	conv = main_lookup_conv(client, sn_norm);
	if (!conv) {
		conv = main_add_conv(client, sn_norm, sn_fmt);
		interface_add_conv(client, conv);
	}

	g_free(sn_norm);
}

EXPORT void
cb_mute_to_conv_toggled(GtkToggleButton *togglebutton, gpointer data)
{
	conv_t *conv;
	client_t *client;

	/* verify we have a selected user and conv */
	client = get_selected_client();
	g_return_if_fail(client != NULL);
	conv = get_selected_conv(client);
	g_return_if_fail(conv != NULL);

	conv->mute_to = gtk_toggle_button_get_active(togglebutton);
}

EXPORT void
cb_mute_from_conv_toggled(GtkToggleButton *togglebutton, gpointer data)
{
	conv_t *conv;
	client_t *client;

	/* verify we have a selected user and conv */
	client = get_selected_client();
	g_return_if_fail(client != NULL);
	conv = get_selected_conv(client);
	g_return_if_fail(conv != NULL);

	conv->mute_from = gtk_toggle_button_get_active(togglebutton);
}

EXPORT void
cb_inject_button_clicked(GtkButton *button, gpointer data)
{
	interface_process_inject();
}

EXPORT void
cb_conv_list_cursor_changed(GtkTreeView *treeview, gpointer data)
{
	client_t *client;
	conv_t *conv;

	g_debug("conv_list_cursor_changed");

	interface_reset_conv();

	/* look up the currently selected row in the client list */
	client = get_selected_client();
	g_return_if_fail(client != NULL);

	/* look up the currently selected row in the conv list */
	conv = get_selected_conv(client);
	g_return_if_fail(conv != NULL);

	interface_set_conv(client, conv);

	/* refresh client statistics */
	interface_refresh_stats(client);

	/* update the msg list for the selected conversation */
	interface_msg_refresh(client, conv);
}

EXPORT void
cb_client_list_cursor_changed(GtkTreeView *treeview, gpointer data)
{
	client_t *client;

	g_debug("client_list_cursor_changed");

	/* clear out all current info */
	interface_reset_all();

	/* look up the currently selected row in the client list */
	client = get_selected_client();
	g_return_if_fail(client != NULL);

	/* update the conv list and other widgets for the current client */
	interface_set_client(client);

	/* refresh client statistics */
	interface_refresh_stats(client);

	/* auto-select the first conv in the list */
	gtk_tree_view_set_cursor(GTK_TREE_VIEW(aj->conv_list), gtk_tree_path_new_first(), NULL, FALSE);
}

void
interface_add_msg(client_t *client, conv_t *conv, msg_t *msg)
{
	GtkTextMark *mark;
	GtkTextIter iter, start_iter, end_iter;

	g_assert(client != NULL);
	g_assert(conv != NULL);
	g_assert(msg != NULL);

	client_t *selected_client = get_selected_client();
	g_return_if_fail(selected_client != NULL);

	if (selected_client == client) {
		/* refresh the statistics window */
		interface_refresh_stats(client);
	}

	conv_t *selected_conv = get_selected_conv(selected_client);
	g_return_if_fail(selected_conv != NULL);

	if (selected_client == client && selected_conv == conv) {
		gtk_text_buffer_get_end_iter(aj->msg_buffer, &iter);
	
		/* insert message into the buffer */
		interface_insert_buffer(client, conv, msg, &iter);
	}

	/* scroll to bottom of text view */
	gtk_text_buffer_get_bounds(aj->msg_buffer, &start_iter, &end_iter);
	mark = gtk_text_buffer_create_mark(aj->msg_buffer, NULL, &end_iter, FALSE);
	gtk_text_view_scroll_mark_onscreen(GTK_TEXT_VIEW(aj->msg_data), mark);
	gtk_text_buffer_delete_mark(aj->msg_buffer, mark);
}

void
interface_add_conv(client_t *client, conv_t *conv)
{
	GtkTreeIter iter;

	g_assert(client != NULL);
	g_assert(conv != NULL);

	client_t *selected_client = get_selected_client();
	g_return_if_fail(selected_client != NULL);

	/* only add the conversation now if the client is the currently selected on */
	if (selected_client == client) {
		gtk_list_store_append(aj->conv_store, &iter);
		gtk_list_store_set(aj->conv_store, &iter, 0, conv->sn_norm, 1, conv, -1);

		/* refresh the statistics window */
		interface_refresh_stats(client);
	}
}

void
interface_destroy_client(client_t *client)
{
	gboolean valid;
	GtkTreeIter iter;
	client_t *selected_client;

	g_assert(client != NULL);

	selected_client = get_selected_client(); 

	valid = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(aj->client_store), &iter);
	while (valid) {
		gchar *sn_norm;

		gtk_tree_model_get(GTK_TREE_MODEL(aj->client_store), &iter, 0, &sn_norm, -1);

		if (g_ascii_strcasecmp(client->sn_norm, sn_norm) == 0) {
			gtk_list_store_remove(aj->client_store, &iter);

			if (client == selected_client) {
				interface_reset_all();
			}

			g_free(sn_norm);
			return;
		}

		g_free(sn_norm);

		valid = gtk_tree_model_iter_next(GTK_TREE_MODEL(aj->client_store), &iter);
	}
}

void
interface_add_client(client_t *client)
{
	GtkTreeIter iter;

	g_assert(client != NULL);

	gtk_list_store_append(aj->client_store, &iter);
	gtk_list_store_set(aj->client_store, &iter, 0, client->sn_norm, 1, client, -1);
}

EXPORT void
cb_config_start_button_clicked(GtkButton *button, gpointer data)
{
	gint ret;
	intf_t *i;
	gboolean spoof;
	gchar *intf_str, *pcap_str = NULL;
	struct intf_entry intf;
	struct addr intf_addr;
	gchar errbuf[PCAP_ERRBUF_SIZE];
	pcap_if_t *d, *devs;
	pcap_addr_t *pcap_addr;

	gtk_widget_hide(aj->config_window);

	/* retrieve selected interface */
	intf_str = gtk_combo_box_get_active_text(GTK_COMBO_BOX(aj->config_interface_entry));
	g_assert(intf_str != NULL);

	ret = pcap_findalldevs(&devs, errbuf);
	if (ret == -1) {
		g_error("pcap findalldevs failed");
	}
	for (d = devs; d; d = d->next) {
		pcap_str = pcap_intf_name(d);
		if (strcmp(pcap_str, intf_str) == 0) {
			break;
		}
	}
	g_assert(d != NULL);
	for (pcap_addr = d->addresses; pcap_addr; pcap_addr = pcap_addr->next) {
		if (pcap_addr->addr->sa_family == AF_INET) {
			break;
		}
	}
	g_assert(pcap_addr != NULL);
	
	ret = addr_ston(pcap_addr->addr, &intf_addr);
	g_assert(ret != -1);

	/* get ip address of selected interface */
	i = intf_open();
	g_assert(i != NULL);
	intf.intf_len = sizeof(struct intf_entry);
	ret = intf_get_src(i, &intf, &intf_addr);
	g_assert(ret != -1);
	memcpy(&aj->intf_pa, &intf.intf_addr, sizeof(struct addr));
	g_debug("%s", addr_ntoa(&intf.intf_addr));
	memcpy(&aj->intf_ha, &intf.intf_link_addr, sizeof(struct addr));
	intf_close(i);

	aj->intf_str_pcap = g_strdup(d->name);
	aj->intf_str_dnet = g_strdup(intf.intf_name);

	/* initialize the connection handler */
	conn_init();

	/* integrated arp/dns spoofing */
	spoof = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(aj->config_spoofing_checkbox));
	if (spoof) {
		g_debug("integrated arp/dns spoofing enabled");

		aj->eth = eth_open(aj->intf_str_dnet);
		g_assert(aj->eth != NULL);

		aj->arp_thread = g_thread_create(arpspoof_init, NULL, TRUE, NULL);
		aj->dns_thread = g_thread_create(dnsspoof_init, NULL, TRUE, NULL);

		g_assert(aj->arp_thread != NULL && aj->dns_thread != NULL);
	}

}

void
interface_init(gint argc, gchar **argv)
{
	gint ret;
	gchar *tmp, *path, *intf;
	GtkTreeIter iter;
	GtkTreeViewColumn *column;
	GtkCellRenderer *renderer;
	gchar errbuf[PCAP_ERRBUF_SIZE];
	pcap_if_t *d, *devs;

	gtk_init(&argc, &argv);

	/* look for glade file */
	aj->xml = NULL;
	tmp = g_get_current_dir();
	path = g_strdup_printf("%s%s", tmp, GLADE);
	if (!aj->xml && g_file_test(path, G_FILE_TEST_EXISTS)) {
		aj->xml = glade_xml_new(path, NULL, NULL);
	}
	g_free(path);
	g_free(tmp);

	path = g_strdup_printf("%s%s", ".." G_DIR_SEPARATOR_S "data", GLADE);
	if (!aj->xml && g_file_test(path, G_FILE_TEST_EXISTS)) {
		aj->xml = glade_xml_new(path, NULL, NULL);
	}
	g_free(path);

	path = g_strdup_printf("%s%s", AIMJECT_DIR, GLADE);
	if (!aj->xml && g_file_test(path, G_FILE_TEST_EXISTS)) {
		aj->xml = glade_xml_new(path, NULL, NULL);
	}
	g_free(path);

	if (!aj->xml) {
		g_error("could not find aimject.glade interface file");
	}

	glade_xml_signal_autoconnect(aj->xml);

	aj->input_window = glade_xml_get_widget(aj->xml, "input_window");
	aj->input_screenname_entry = glade_xml_get_widget(aj->xml, "input_screenname_entry");

	aj->config_window = glade_xml_get_widget(aj->xml, "config_window");
	aj->config_interface_entry = glade_xml_get_widget(aj->xml, "config_interface_entry");
	aj->config_spoofing_checkbox = glade_xml_get_widget(aj->xml, "config_spoofing_checkbox");

	gtk_combo_box_remove_text(GTK_COMBO_BOX(aj->config_interface_entry), 0);

	ret = pcap_findalldevs(&devs, errbuf);
	if (ret == -1) {
		g_error("pcap findalldevs failed");
	}
	for (d = devs; d; d = d->next) {
		if (d->flags & PCAP_IF_LOOPBACK) {
			continue;
		}
		if (strcmp(d->name, "any") == 0) {
			continue;
		}

		intf = pcap_intf_name(d);
		gtk_combo_box_append_text(GTK_COMBO_BOX(aj->config_interface_entry), intf);
	}
	gtk_combo_box_set_active(GTK_COMBO_BOX(aj->config_interface_entry), 0);

	aj->main_window = glade_xml_get_widget(aj->xml, "main_window");
	aj->client_list = glade_xml_get_widget(aj->xml, "client_list");
	aj->conv_list = glade_xml_get_widget(aj->xml, "conv_list");
	aj->stats_list = glade_xml_get_widget(aj->xml, "stats_list");
	aj->msg_data = glade_xml_get_widget(aj->xml, "msg_data");
	aj->inject_data = glade_xml_get_widget(aj->xml, "inject_data");
	aj->inject_button = glade_xml_get_widget(aj->xml, "inject_button");
	aj->inject_radio1 = glade_xml_get_widget(aj->xml, "inject_radio1");
	aj->inject_radio2 = glade_xml_get_widget(aj->xml, "inject_radio2");
	aj->add_conv_button = glade_xml_get_widget(aj->xml, "add_conv_button");
	aj->delete_conv_button = glade_xml_get_widget(aj->xml, "delete_conv_button");
	aj->mute_to_conv_button = glade_xml_get_widget(aj->xml, "mute_to_conv_button");
	aj->mute_from_conv_button = glade_xml_get_widget(aj->xml, "mute_from_conv_button");

	aj->msg_buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(aj->msg_data));
	gtk_text_buffer_create_tag(aj->msg_buffer, "bold", "weight", PANGO_WEIGHT_BOLD, NULL);
	gtk_text_buffer_create_tag(aj->msg_buffer, "blue_foreground", "foreground", "#16569E", NULL);
	gtk_text_buffer_create_tag(aj->msg_buffer, "red_foreground", "foreground", "#A82F2F", NULL);

	aj->inject_buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(aj->inject_data));

	/* set up the client list */
	aj->client_store = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_POINTER);
	gtk_tree_view_set_model(GTK_TREE_VIEW(aj->client_list), GTK_TREE_MODEL(aj->client_store));
	g_object_unref(G_OBJECT(aj->client_store));

	renderer = gtk_cell_renderer_text_new();
	column = gtk_tree_view_column_new_with_attributes("Local ID", renderer, "text", 0, NULL);
	gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
	gtk_tree_view_column_set_fixed_width(column, 80);
	gtk_tree_view_append_column(GTK_TREE_VIEW(aj->client_list), column);

	g_signal_connect(aj->client_list, "cursor-changed", (GCallback) cb_client_list_cursor_changed, NULL);

	/* set up the conversation list */
	aj->conv_store = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_POINTER);
	gtk_tree_view_set_model(GTK_TREE_VIEW(aj->conv_list), GTK_TREE_MODEL(aj->conv_store));
	g_object_unref(G_OBJECT(aj->conv_store));

	renderer = gtk_cell_renderer_text_new();
	column = gtk_tree_view_column_new_with_attributes("Remote ID", renderer, "text", 0, NULL);
	gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
	gtk_tree_view_column_set_fixed_width(column, 80);
	gtk_tree_view_append_column(GTK_TREE_VIEW(aj->conv_list), column);

	g_signal_connect(aj->conv_list, "cursor-changed", (GCallback) cb_conv_list_cursor_changed, NULL);

	/* set up the stats list */
	aj->stats_store = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_STRING);
	gtk_tree_view_set_model(GTK_TREE_VIEW(aj->stats_list), GTK_TREE_MODEL(aj->stats_store));
	g_object_unref(G_OBJECT(aj->stats_store));

	renderer = gtk_cell_renderer_text_new();
	column = gtk_tree_view_column_new_with_attributes("Statistic", renderer, "text", 0, NULL);
	gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
	gtk_tree_view_column_set_fixed_width(column, 100);
	gtk_tree_view_append_column(GTK_TREE_VIEW(aj->stats_list), column);

	renderer = gtk_cell_renderer_text_new();
	column = gtk_tree_view_column_new_with_attributes("Value", renderer, "text", 1, NULL);
	gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
	gtk_tree_view_column_set_fixed_width(column, 120);
	gtk_tree_view_append_column(GTK_TREE_VIEW(aj->stats_list), column);

	gtk_list_store_append(aj->stats_store, &iter);
	gtk_list_store_set(aj->stats_store, &iter, 0, "Initial Conn", 1, "", -1);
	gtk_list_store_append(aj->stats_store, &iter);
	gtk_list_store_set(aj->stats_store, &iter, 0, "BOS Conn", 1, "", -1);
	gtk_list_store_append(aj->stats_store, &iter);
	gtk_list_store_set(aj->stats_store, &iter, 0, "Conv Count", 1, "", -1);
	gtk_list_store_append(aj->stats_store, &iter);
	gtk_list_store_set(aj->stats_store, &iter, 0, "Msg Recv Count", 1, "", -1);
	gtk_list_store_append(aj->stats_store, &iter);
	gtk_list_store_set(aj->stats_store, &iter, 0, "Msg Sent Count", 1, "", -1);
	gtk_list_store_append(aj->stats_store, &iter);
	gtk_list_store_set(aj->stats_store, &iter, 0, "Last Recv Msg", 1, "", -1);
	gtk_list_store_append(aj->stats_store, &iter);
	gtk_list_store_set(aj->stats_store, &iter, 0, "Last Sent Msg", 1, "", -1);
	gtk_list_store_append(aj->stats_store, &iter);
	gtk_list_store_set(aj->stats_store, &iter, 0, "Sequence In", 1, "", -1);
	gtk_list_store_append(aj->stats_store, &iter);
	gtk_list_store_set(aj->stats_store, &iter, 0, "Sequence Out", 1, "", -1);
	gtk_list_store_append(aj->stats_store, &iter);
	gtk_list_store_set(aj->stats_store, &iter, 0, "Conv SN Format", 1, "", -1);
	gtk_list_store_append(aj->stats_store, &iter);
	gtk_list_store_set(aj->stats_store, &iter, 0, "Conv Font Style", 1, "", -1);

	gtk_tree_selection_set_mode(gtk_tree_view_get_selection(GTK_TREE_VIEW(aj->stats_list)), GTK_SELECTION_NONE);

	gtk_widget_show(aj->main_window);
}
