/*
 * Aimject
 *
 * Copyright (C) 2006 Jon Oberheide <jon@oberheide.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <config.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <glib.h>

#include "win32.h"
#include "main.h"

extern aimject_t *aj;

/* OS-specific setup */
void
os_init(void)
{
	int err;
	WSADATA wsaData;
	WORD wVersionRequested;

	wVersionRequested = MAKEWORD(2, 2);
	err = WSAStartup(wVersionRequested, &wsaData);
	if (err != 0) {
		g_error("error initializing winsock");
	}

	signal(SIGINT, signal_cleanup);
	signal(SIGTERM, signal_cleanup);
}

/* pcap interface formatting */
gchar *
pcap_intf_name(pcap_if_t *dev)
{
	return dev->description;
}

/* ip forwarding */
gboolean
enable_ip_forwarding(void)
{
	arp_t *a;
	struct arp_entry arp_e;
	long ret;
	HKEY handle;
	DWORD dim = 4;
	DWORD value;

	/* add a static arp entry for the gateway */
	a = arp_open();
	g_assert(a != NULL);
	memcpy(&arp_e.arp_pa, &aj->gw_pa, sizeof(struct addr));
	memcpy(&arp_e.arp_ha, &aj->gw_ha, sizeof(struct addr));
	arp_add(a, &arp_e);
	arp_close(a);

	/* get registry handle */
	RegOpenKeyEx(HKEY_LOCAL_MACHINE,
			 TEXT("SYSTEM\\CurrentControlSet\\Services\\Tcpip\\Parameters"),
			 0, KEY_SET_VALUE, &handle);

	/* enable ip forwarding */
	value = 1;
	ret = RegSetValueEx(handle, TEXT("IPEnableRouter"), 0, REG_DWORD, (LPBYTE) &value, dim);
	if (ret != 0) {
		return FALSE;
	}

	/* disable icmp redirects */
	value = 0;
	ret = RegSetValueEx(handle, TEXT("EnableICMPRedirect"), 0, REG_DWORD, (LPBYTE) &value, dim);
	if (ret != 0) {
		return FALSE;
	}

	RegCloseKey(handle);

	return TRUE;
}

/* dn_expand() */
#ifndef INDIR_MASK
#define INDIR_MASK  0xc0
#endif

gint
dn_expand(guchar *msg, guchar *eom_orig, guchar *comp_dn, guchar *exp_dn, gint length)
{
	guchar *cp, *dn, *eom;
	gint c, n, len = -1, checked = 0;

	dn = exp_dn;
	cp = comp_dn;
	eom = exp_dn + length;

	while ((n = *cp++) != 0) {
		switch (n & INDIR_MASK) {
			case 0:
			if (dn != exp_dn) {
				if (dn >= eom)
					return (-1);
				*dn++ = '.';
			}
			if (dn+n >= eom)
				return (-1);
			checked += n + 1;
			while (--n >= 0) {
				int c = *cp++;
				if ((c == '.') || (c == '\\')) {
					if (dn + n + 2 >= eom)
						return (-1);
					*dn++ = '\\';
				}
				*dn++ = c;
				if (cp >= eom_orig)
					return (-1);
			}
			break;

			case INDIR_MASK:
			if (len < 0)
				len = cp - comp_dn + 1;
			cp = msg + (((n & 0x3f) << 8) | (*cp & 0xff));
			if (cp < msg || cp >= eom_orig)
				return (-1);
			checked += 2;
			if (checked >= eom_orig - msg)
				return (-1);
			break;

			default:
			return (-1);
		}
	}

	*dn = '\0';
	for (dn = exp_dn; (c = *dn) != '\0'; dn++)
		if (isascii(c) && isspace(c))
			return (-1);
		
	if (len < 0)
		len = cp - comp_dn;
	return (len);
}
